package com.zanxiang.game.back.serve.utils;

import com.github.sd4324530.jtuple.Tuple2;
import com.github.sd4324530.jtuple.Tuples;
import com.zanxiang.game.back.serve.pojo.entity.GameBackPolicy;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.NumberUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Objects;
import java.util.function.Function;

@Slf4j
public class BackPolicyUtil {
    private static final long[] RECHARGE_LEVEL = {12 * 100L, 30 * 100L, 50 * 100L, 98 * 100L, 198 * 100L, 328 * 100L, 649 * 100L, 998 * 100L, 2998 * 100L};

    static {
        Arrays.sort(RECHARGE_LEVEL);
    }

    public static void main(String[] args) {
        GameBackPolicy backPolicy = GameBackPolicy.builder()
                .id(8L)
                .backPolicyName("只传首充")
                .firstMinMoney(3000L)
                .firstMinMoneyType(0)
                .firstMinMoneyRate("1:0")
                .firstMaxMoney(19800L)
                .firstMaxMoneyType(0)
                .firstMaxMoneyRate("1:0")
                .firstOtherMoneyType(0)
                .firstOtherMoneyRate("1:0")
                .rechargeMinMoney(3000L)
                .rechargeMinMoneyType(1)
                .rechargeMinMoneyRate("0")
                .rechargeMaxMoney(19800L)
                .rechargeMaxMoneyType(1)
                .rechargeMaxMoneyRate("0")
                .rechargeOtherMoneyType(1)
                .rechargeOtherMoneyRate("0")
                .levelDown(0)
                .build();
        String orderId = "2023072623022469103393434";
        long money = 800;
        System.out.println(backOrder(orderId, backPolicy, money, () -> {
            return true;
        }, (count) -> {
            return 0;
        }));
    }

    /**
     * 订单回传
     *
     * @param gameBackPolicy   回传策略
     * @param rechargeMoney    充值金额
     * @param lastBackFunction 在指定最近订单数量内是否有订单回传过
     * @return <是否回传, 回传金额>
     */
    public static boolean backOrder(String orderId, GameBackPolicy gameBackPolicy, long rechargeMoney, FirstOrder firstOrder, Function<Integer, Integer> lastBackFunction) {
        if (null == gameBackPolicy) {
            log.error("订单 id[{}]通过回传策略[null]判断后，是否回传：{}.", orderId, true);
            return true;
        }
        log.error("----orderId: {}, backPolicy: {}, money: {}", orderId, JsonUtil.toString(gameBackPolicy), rechargeMoney);
        Integer firstType;
        String firstRate;
        Integer rechargeType;
        String rechargeRate;
        if (rechargeMoney <= gameBackPolicy.getFirstMinMoney()) {
            // 小于首充金额
            firstType = gameBackPolicy.getFirstMinMoneyType();
            firstRate = gameBackPolicy.getFirstMinMoneyRate();
        } else if (rechargeMoney >= gameBackPolicy.getFirstMaxMoney()) {
            firstType = gameBackPolicy.getFirstMaxMoneyType();
            firstRate = gameBackPolicy.getFirstMaxMoneyRate();
        } else {
            firstType = gameBackPolicy.getFirstOtherMoneyType();
            firstRate = gameBackPolicy.getFirstOtherMoneyRate();
        }
        if (rechargeMoney <= gameBackPolicy.getRechargeMinMoney()) {
            rechargeType = gameBackPolicy.getRechargeMinMoneyType();
            rechargeRate = gameBackPolicy.getRechargeMinMoneyRate();
        } else if (rechargeMoney >= gameBackPolicy.getRechargeMaxMoney()) {
            rechargeType = gameBackPolicy.getRechargeMaxMoneyType();
            rechargeRate = gameBackPolicy.getRechargeMaxMoneyRate();
        } else {
            rechargeType = gameBackPolicy.getRechargeOtherMoneyType();
            rechargeRate = gameBackPolicy.getRechargeOtherMoneyRate();
        }
        Integer backType;
        String backRate;
        if (firstType.equals(rechargeType) && firstRate.equalsIgnoreCase(rechargeRate)) {
            // 回传内容相同，不需要区分是不是首单
            backType = firstType;
            backRate = firstRate;
        } else {
            if (firstOrder.isFirstOrder()) {
                log.error("订单{}是首单", orderId);
                backType = firstType;
                backRate = firstRate;
            } else {
                log.error("订单{}不是首单", orderId);
                backType = rechargeType;
                backRate = rechargeRate;
            }
        }
        if (Objects.equals(backType, GameBackPolicy.POLICY_TYPE_FIXED_RATE)) {
            // 固定比例回传
            String[] temp = backRate.split(":");
            int backCount = Integer.parseInt(temp[0]);
            int ignoreCount = Integer.parseInt(temp[1]);
            if (backCount == 0) {
                log.error("订单 id[{}]通过回传策略[{}]判断后，是否回传：{}.", orderId, gameBackPolicy.getId(), false);
                return false;
            }
            if (ignoreCount == 0) {
                log.error("订单 id[{}]通过回传策略[{}]判断后，是否回传：{}.", orderId, gameBackPolicy.getId(), true);
                return true;
            }
            int isBackCount = lastBackFunction.apply(backCount + ignoreCount - 1);
            log.error("订单 id[{}]通过回传策略[{}]判断后，是否回传：{}.", orderId, gameBackPolicy.getId(), (isBackCount < backCount));
            return isBackCount < backCount;
        } else if (Objects.equals(backType, GameBackPolicy.POLICY_TYPE_RANDOM_RATE)) {
            // 随机概率回传
            boolean isBack = RandomUtils.nextInt(0, 100) < NumberUtil.multiply100(new BigDecimal(backRate)).intValue();
            log.error("订单 id[{}]通过回传策略[{}]判断后，是否回传：{}.", orderId, gameBackPolicy.getId(), isBack);
            return isBack;
        } else {
            throw new RuntimeException("回传策略[" + gameBackPolicy.getId() + "]配置错误，未知的回传类型：" + backType);
        }
    }

    private static Tuple2<Integer, Integer> approximate(int molecular, int denominator) {
        if (molecular % denominator == 0) {
            int temp = molecular / denominator;
            return Tuples.tuple(temp, temp);
        }
        int model = 2;
        do {
            if (molecular % model == 0 && denominator % model == 0) {
                molecular = molecular / model;
                denominator = denominator / model;
                continue;
            }
            model++;
        } while (model <= denominator / 2);
        return Tuples.tuple(molecular, denominator);
    }

    /**
     * 降档
     */
    public static long lowRechargeLevel(long rechargeMoney, Integer levelDown) {
        if (levelDown == null || levelDown == 0) {
            return rechargeMoney;
        }
        int level = RECHARGE_LEVEL.length - 1;
        for (int i = 0; i < RECHARGE_LEVEL.length; i++) {
            if (rechargeMoney <= RECHARGE_LEVEL[i]) {
                level = i;
                break;
            }
        }
        level = level - levelDown;
        level = level < 0 ? 0 : Math.min(level, RECHARGE_LEVEL.length - 1);
        return RECHARGE_LEVEL[level];
    }

    public interface FirstOrder {
        boolean isFirstOrder();
    }
}
